require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_resourcecontroller',
  dscmeta_resource_friendly_name: 'ResourceController',
  dscmeta_resource_name: 'ResourceController',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'ResourceControllerDSC',
  dscmeta_module_version: '2.0.1',
  docs: 'The DSC ResourceController resource type.
         Automatically generated from version 2.0.1',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_instancename: {
      type: 'String',
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_suppressreboot: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_resourceversion: {
      type: 'String',
      desc: ' ',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_result: {
      type: 'Optional[Array[Struct[{
              key => Optional[String],
              value => Optional[String],
            }]]]',
      desc: ' ',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_KeyValuePair[]',
      mof_is_embedded: true,
    },
    dsc_resourcename: {
      type: 'String',
      desc: ' ',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_properties: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_maintenancewindow: {
      type: "Optional[Array[Struct[{
              frequency => Optional[String],
              day => Optional[Array[Enum['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30', '31']]],
              dayofweek => Optional[Array[Enum['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday']]],
              startdate => Optional[Timestamp],
              enddate => Optional[Timestamp],
              starttime => Optional[Timestamp],
              endtime => Optional[Timestamp],
              week => Optional[Array[Enum['0', '1', '2', '3', '4']]]
            }]]]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MaintenanceWindow[]',
      mof_is_embedded: true,
    },
    dsc_credentials: {
      type: 'Optional[Array[Struct[{
              credential => Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]],
              name => String,
            }]]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Credential[]',
      mof_is_embedded: true,
    },
  },
)
